/*
 * Jagg Directory Anaylzer
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.beans;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.util.LinkedList;


public class AsynchronousPropertyChangeSupport extends PropertyChangeSupport {
    protected Object sourceBean;
    protected PropertyChangeSupport internalPropertyChangeSupport;
    protected InternalListener internalListener;
    protected LinkedList eventQueue;
    protected Thread externalNotifierThread;

    public AsynchronousPropertyChangeSupport(Object sourceBean) {
        super(sourceBean);
        this.sourceBean = sourceBean;
        internalPropertyChangeSupport = new PropertyChangeSupport(sourceBean);
        internalListener = new InternalListener(this);
        super.addPropertyChangeListener(internalListener);
        eventQueue = new LinkedList();
    }

    protected void manageThread() {
        PropertyChangeListener[] propertyChangeListeners =
            internalPropertyChangeSupport.getPropertyChangeListeners();

        if (
            (externalNotifierThread == null)
                && (propertyChangeListeners.length > 0)) {
            ExternalNotifier externalNotifier = new ExternalNotifier(this);
            externalNotifierThread = new Thread(externalNotifier);
            externalNotifierThread.setPriority(8);
            externalNotifierThread.start();
        }

        if (
            (externalNotifierThread != null)
                && (propertyChangeListeners.length <= 0)) {
            externalNotifierThread.interrupt();
            externalNotifierThread = null;
        }
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        internalPropertyChangeSupport.addPropertyChangeListener(listener);
        manageThread();
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        internalPropertyChangeSupport.removePropertyChangeListener(listener);
        manageThread();
    }

    public PropertyChangeListener[] getPropertyChangeListeners() {
        return internalPropertyChangeSupport.getPropertyChangeListeners();
    }

    public void addPropertyChangeListener(
        String propertyName, PropertyChangeListener listener) {
        internalPropertyChangeSupport.addPropertyChangeListener(
            propertyName, listener);
        manageThread();
    }

    public void removePropertyChangeListener(
        String propertyName, PropertyChangeListener listener) {
        internalPropertyChangeSupport.removePropertyChangeListener(
            propertyName, listener);
        manageThread();
    }

    public PropertyChangeListener[] getPropertyChangeListeners(
        String propertyName) {
        return internalPropertyChangeSupport.getPropertyChangeListeners(
            propertyName);
    }

    public boolean hasListeners(String propertyName) {
        return internalPropertyChangeSupport.hasListeners(propertyName);
    }

    protected class InternalListener implements PropertyChangeListener {
        protected AsynchronousPropertyChangeSupport asynchronousPropertyChangeSupport;

        public InternalListener(
            AsynchronousPropertyChangeSupport asynchronousPropertyChangeSupport) {
            this.asynchronousPropertyChangeSupport =
                asynchronousPropertyChangeSupport;
        }

        public void propertyChange(PropertyChangeEvent evt) {
            if (hasListeners(evt.getPropertyName())) {
                synchronized (asynchronousPropertyChangeSupport.eventQueue) {
                    asynchronousPropertyChangeSupport.eventQueue.addLast(evt);
                    asynchronousPropertyChangeSupport.eventQueue.notifyAll();
                }
            } else {
            }
        }
    }

    protected class ExternalNotifier implements Runnable {
        protected AsynchronousPropertyChangeSupport asynchronousPropertyChangeSupport;

        public ExternalNotifier(
            AsynchronousPropertyChangeSupport asynchronousPropertyChangeSupport) {
            this.asynchronousPropertyChangeSupport =
                asynchronousPropertyChangeSupport;
        }

        public void run() {
            try {
                while (true) {
                    synchronized (asynchronousPropertyChangeSupport.eventQueue) {
                        if (
                            asynchronousPropertyChangeSupport.eventQueue
                                .isEmpty()) {
                            asynchronousPropertyChangeSupport.eventQueue.wait();
                        }
                    }

                    while (
                        !asynchronousPropertyChangeSupport.eventQueue.isEmpty()) {
                        PropertyChangeEvent propertyChangeEvent =
                            (PropertyChangeEvent) asynchronousPropertyChangeSupport.eventQueue
                            .removeFirst();

                        asynchronousPropertyChangeSupport.internalPropertyChangeSupport
                        .firePropertyChange(propertyChangeEvent);
                    }
                }
            } catch (InterruptedException e) {
            }
        }
    }
}
