/*
 * Jagg Directory Anaylzer
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jagg.swingui;

import net.chaosserver.io.FileMetaData;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.File;

import java.net.URL;

import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;


public class DirectoryAnalyzer implements PropertyChangeListener {
    protected JFrame directoryAnalyzerFrame;
    protected DirectoryTable directoryTable;
    protected StatusBar statusBar;
    protected ToolBar toolBar;

    public DirectoryAnalyzer() {
    }

    public static void main(String[] args) {
        DirectoryAnalyzer directoryAnalyzer = new DirectoryAnalyzer();
        directoryAnalyzer.start();
    }

    protected void updateFrameTitle(String currentDirectory) {
        if (directoryAnalyzerFrame != null) {
            String frameTitle = getFrameTitle();

            if (currentDirectory != null) {
                frameTitle = currentDirectory + " - " + frameTitle;
            }

            directoryAnalyzerFrame.setTitle(frameTitle);
        }
    }

    protected String getFrameTitle() {
        Package currentPackage = getClass().getPackage();
        String specificationTitle = currentPackage.getSpecificationTitle();
        String specificationVersion = currentPackage.getSpecificationVersion();
        String implementationVersion =
            currentPackage.getImplementationVersion();

        StringBuffer sb = new StringBuffer();

        if (specificationTitle != null) {
            sb.append(specificationTitle);
        } else {
            sb.append("DirectoryAnalyzer");
        }

        if (specificationVersion != null) {
            sb.append(" ");
            sb.append(specificationVersion);
        }

        if (implementationVersion != null) {
            sb.append(" {Build ID: ");
            sb.append(implementationVersion);
            sb.append("}");
        }

        return sb.toString();
    }

    public void start() {
        directoryAnalyzerFrame = new JFrame(getFrameTitle());

        // Set the icon image
        URL frameImageUrl =
            getClass().getResource("/net/chaosserver/jagg/swingui/jagg.gif");
        Toolkit toolkit = directoryAnalyzerFrame.getToolkit();
        Image frameImage = toolkit.createImage(frameImageUrl);
        directoryAnalyzerFrame.setIconImage(frameImage);

        File startDirectory = getStartDirectory(directoryAnalyzerFrame);

        BorderLayout borderLayout = new BorderLayout();
        directoryAnalyzerFrame.getContentPane().setLayout(borderLayout);

        WindowCloser windowCloser = new WindowCloser(this);
        directoryAnalyzerFrame.addWindowListener(windowCloser);

        statusBar = new StatusBar(directoryTable);
        toolBar = new ToolBar();

        directoryAnalyzerFrame.getContentPane().add(
            toolBar, BorderLayout.NORTH);
        directoryAnalyzerFrame.getContentPane().add(
            statusBar, BorderLayout.SOUTH);

        try {
            updateFrameTitle(startDirectory.getCanonicalPath());
        } catch (Exception e) {
        }

        directoryAnalyzerFrame.setSize(getStartSize());
        directoryAnalyzerFrame.setLocation(getStartLocation());
        directoryAnalyzerFrame.setVisible(true);

        // Add the main panel after showing something to the user.
        directoryTable = new DirectoryTable(startDirectory);
        directoryTable.addPropertyChangeListener(this);
        statusBar.setDirectoryTable(directoryTable);

        JScrollPane scrollPane = new JScrollPane(directoryTable);
        directoryAnalyzerFrame.getContentPane().add(
            scrollPane, BorderLayout.CENTER);

        directoryAnalyzerFrame.validate();
    }

    protected File getStartDirectory(Component parent) {
        File startDirectory = new File(".");

        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Select Starting Directory");
        chooser.setApproveButtonText("Select");
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        int returnVal = chooser.showOpenDialog(parent);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            startDirectory = chooser.getSelectedFile();
        }

        return startDirectory;
    }

    public void stop() {
        if (directoryTable != null) {
            directoryTable.removePropertyChangeListener(this);
        }

        saveWindowPosition();
        System.exit(0);
    }

    protected Point getStartLocation() {
        Preferences preferences = Preferences.userNodeForPackage(getClass());

        Point startLocation =
            new Point(preferences.getInt("x", 10), preferences.getInt("y", 10));

        return startLocation;
    }

    protected Dimension getStartSize() {
        Preferences preferences = Preferences.userNodeForPackage(getClass());

        Dimension startSize =
            new Dimension(
                preferences.getInt("width", 480),
                preferences.getInt("height", 640));

        return startSize;
    }

    protected void saveWindowPosition() {
        Preferences preferences = Preferences.userNodeForPackage(getClass());

        Point location = directoryAnalyzerFrame.getLocation();
        Dimension size = directoryAnalyzerFrame.getSize();

        preferences.putInt("height", (int) size.getHeight());
        preferences.putInt("width", (int) size.getWidth());

        preferences.putInt("x", (int) location.getX());
        preferences.putInt("y", (int) location.getY());
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if (
            DirectoryTable.FILE_META_DATA_PROPERTY.equals(
                    evt.getPropertyName())) {
            FileMetaData currentFileMetaData = (FileMetaData) evt.getNewValue();
            updateFrameTitle(currentFileMetaData.getFullName());
        }
    }

    protected class WindowCloser extends WindowAdapter {
        protected DirectoryAnalyzer directoryAnalyzer;

        public WindowCloser(DirectoryAnalyzer directoryAnalyzer) {
            this.directoryAnalyzer = directoryAnalyzer;
        }

        public void windowClosing(WindowEvent evt) {
            directoryAnalyzer.stop();
        }
    }
}
