/*
 * Jagg Directory Anaylzer
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.jagg.swingui;

import net.chaosserver.io.FileMetaData;

import net.chaosserver.util.PropertyComparator;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.io.File;

import java.text.DecimalFormat;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.table.AbstractTableModel;


public class DirectoryTableModel extends AbstractTableModel
    implements PropertyChangeListener {
    public static final String NAME = "Name";
    public static final String SIZE = "Size";
    public static final String TYPE = "Type";
    public static final int SPECIAL_ROWS = 1;
    protected FileMetaData fileMetaData;
    protected List childMetaDataList;
    protected DecimalFormat decimalFormat = new DecimalFormat();
    protected PropertyChangeSupport propertyChangeSupport =
        new PropertyChangeSupport(this);

    public DirectoryTableModel(File currentDirectory) {
        setFileMetaData(new FileMetaData(currentDirectory));
    }

    /**
     * Setter for the fileMetaData property
     *
     * @param fileMetaData the new value of the fileMetaData property
     * @see #getFileMetaData
     */
    protected void setFileMetaData(FileMetaData fileMetaData) {
        FileMetaData oldFileMetaData = this.fileMetaData;

        if (this.fileMetaData != null) {
            if (!this.fileMetaData.isStateTransitioning()) {
                this.fileMetaData.removePropertyChangeListener(this);
            }
        }

        this.fileMetaData = fileMetaData;

        if (this.fileMetaData != null) {
            this.fileMetaData.addPropertyChangeListener(this);
        }

        childMetaDataList =
            new LinkedList(fileMetaData.getChildMetaData().values());
        Collections.sort(
            childMetaDataList, new PropertyComparator("length", false));

        fireTableDataChanged();
        propertyChangeSupport.firePropertyChange(
            DirectoryTable.FILE_META_DATA_PROPERTY, oldFileMetaData,
            this.fileMetaData);
    }

    /**
     * Getter for the fileMetaData property
     *
     * @return the value of the fileMetaData property
     * @see #setFileMetaData
     */
    protected FileMetaData getFileMetaData() {
        return this.fileMetaData;
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object result = "ERROR";

        switch (rowIndex) {
        case 0:

            switch (columnIndex) {
            case 0:
                result =
                    ".. - " + getFileMetaData().getParentMetaDataLoadState();

                break;

            case 1:
                result = "";

                break;

            case 2:
                result = "Parent File Folder";

                break;
            }

            break;

        default:
            result = getFileMetaDataAt(rowIndex - SPECIAL_ROWS, columnIndex);

            break;
        }

        return result;
    }

    public Object getFileMetaDataAt(int rowIndex, int columnIndex) {
        FileMetaData childFileMetaData =
            (FileMetaData) childMetaDataList.get(rowIndex);

        switch (columnIndex) {
        case 0:
            return childFileMetaData.getName();

        case 1:
            return decimalFormat.format(childFileMetaData.length());

        case 2:
            return childFileMetaData.isDirectory() ? "File Folder" : "File";

        default:
            return "ERROR";
        }
    }

    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
        case 0:
            return NAME;

        case 1:
            return SIZE;

        case 2:
            return TYPE;

        default:
            return "ERROR";
        }
    }

    public int getColumnCount() {
        return 3;
    }

    public int getRowCount() {
        return childMetaDataList.size() + SPECIAL_ROWS;
    }

    public void buildToChild(int rowIndex) {
        switch (rowIndex) {
        case 0:

            if (getFileMetaData().isParentMetaDataLoaded()) {
                FileMetaData parentFileMetaData =
                    getFileMetaData().getParentMetaData();
                setFileMetaData(parentFileMetaData);
            } else {
                getFileMetaData().loadParentMetaDataBackground();
            }

            break;

        default:

            FileMetaData childFileMetaData =
                (FileMetaData) childMetaDataList.get(rowIndex - SPECIAL_ROWS);

            if (childFileMetaData.isDirectory()) {
                setFileMetaData(childFileMetaData);
            }

            break;
        }
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        propertyChangeSupport.firePropertyChange(evt);

        if (
            FileMetaData.PARENT_META_DATA_LOADED_PROPERTY.equals(
                    evt.getPropertyName())) {
            String newState = (String) evt.getNewValue();
            FileMetaData fileMetaDataSource = (FileMetaData) evt.getSource();

            if (FileMetaData.PARENT_METADATA_LOAD_STATE_LOADED.equals(newState)) {
                if (fileMetaDataSource.equals(getFileMetaData())) {
                    // If the user is currently looking at the fileMetaData that
                    // triggered this load, update the display.
                    FileMetaData parentFileMetaData =
                        getFileMetaData().getParentMetaData();
                    setFileMetaData(parentFileMetaData);
                } else {
                    // If the user has gone else where, stop watching
                    fileMetaDataSource.removePropertyChangeListener(this);
                }
            }
        }
    }
}
