/*
 * Jagg Directory Anaylzer
 *
 * Copyright (C) 2002-2003, Jordan Reed <reedjd@bitsmart.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package net.chaosserver.util;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.Comparator;


public class PropertyComparator implements Comparator {
    protected String propertyName;
    protected boolean ascending;

    public PropertyComparator(String propertyName) {
        this(propertyName, true);
    }

    public PropertyComparator(String propertyName, boolean ascending) {
        setPropertyName(propertyName);
        setAscending(ascending);
    }

    /**
     * Setter for the ascending property
     *
     * @param ascending the new value of the ascending property
     */
    protected void setAscending(boolean ascending) {
        this.ascending = ascending;
    }

    /**
     * Getter for the ascending property
     *
     * @return the value of the ascending property
     * @see #setAscending
     */
    protected boolean isAscending() {
        return this.ascending;
    }

    /**
     * Setter for the propertyName property
     *
     * @param propertyName the new value of the propertyName property
     * @see #getPropertyName
     */
    protected void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }

    /**
     * Getter for the propertyName property
     *
     * @return the value of the propertyName property
     * @see #setPropertyName
     */
    protected String getPropertyName() {
        return this.propertyName;
    }

    public int compare(Object o1, Object o2) {
        try {
            PropertyDescriptor propertyDescriptor1 =
                new PropertyDescriptor(getPropertyName(), o1.getClass());

            PropertyDescriptor propertyDescriptor2 =
                new PropertyDescriptor(getPropertyName(), o2.getClass());

            Method readMethod1 = propertyDescriptor1.getReadMethod();
            Method readMethod2 = propertyDescriptor2.getReadMethod();

            Comparable property1 =
                (Comparable) readMethod1.invoke(o1, new Object[] {  });
            Comparable property2 =
                (Comparable) readMethod2.invoke(o2, new Object[] {  });

            int result = property1.compareTo(property2);

            if (!isAscending()) {
                result *= -1;
            }

            return result;
        } catch (IntrospectionException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    public boolean equals(Object object) {
        return object == this;
    }
}
